"""
Provide support for Cached loader.
Generate key for cached templates based on template names
and device type (desktop or mobile or tablet)
"""
from django.template.loaders.cached import Loader as CachedLoader
from django_adaptive.loader_utils import get_template_suffix


class AdaptiveTemplateCache(dict):

    def get_device(self):
        return get_template_suffix()

    def get_key(self, key):
        return key + self.get_device()

    def __getitem__(self, key):
        return super(AdaptiveTemplateCache, self).__getitem__(
            self.get_key(key))

    def __setitem__(self, key, value):
        return super(AdaptiveTemplateCache, self).__setitem__(
            self.get_key(key), value)


class Loader(CachedLoader):

    def __init__(self, loaders):
        super(Loader, self).__init__(loaders)
        self.template_cache = AdaptiveTemplateCache()



from django.template.loaders.filesystem import Loader as FilesystemLoader
from django.conf import settings

from .loader_utils import get_template_dirs


class Loader(FilesystemLoader):
    def get_template_sources(self, template_name, template_dirs=None):
        if template_dirs is None:
            template_dirs = settings.TEMPLATE_DIRS

        dirs = get_template_dirs(template_dirs)

        return super(Loader, self).get_template_sources(template_name, dirs)



from django.template.loaders.app_directories import Loader \
    as AppDirectoriesLoader, app_template_dirs

from .loader_utils import get_template_dirs


class Loader(AppDirectoriesLoader):
    def get_template_sources(self, template_name, template_dirs=None):
        if template_dirs is None:
            template_dirs = app_template_dirs

        dirs = get_template_dirs(template_dirs)

        return super(Loader, self).get_template_sources(template_name, dirs)



import threading

_thread_locals = threading.local()


def get_current_request():
    return getattr(_thread_locals, 'request', None)


def set_current_request(request):
    _thread_locals.request = request


class ThreadLocals(object):
    """
    Middleware that gets various objects from the
    request object and saves them in thread local storage.
    """
    def process_request(self, request):
        set_current_request(request)


#! /usr/bin/env python
from distutils.core import setup
from setuptools import find_packages
import sys
reload(sys).setdefaultencoding('Utf-8')


setup(
    name='django-adaptive',
    version='1.0.0',
    author='Guillaume Pousseo <guillaumepousseo@revsquare.com>,\
        Tomasz Roszko <tomaszroszko@revsquare.com>,\
        Rafal Selewonko <rafalselewonko@revsquare.com>',
    description='Manages to load templates according to the type of device to\
        a specific directory prefix.',
    long_description=open('README.rst').read(),
    url='http://www.revsquare.com',
    license='BSD License',
    platforms=['OS Independent'],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    install_requires=[
        'Django>=1.6',
        'django-mobi>=0.1.7',
    ],
)



import os

from django.conf import settings
from .middleware import get_current_request

""" Setup constants with directories location prefixes, by default """
DESKTOP = getattr(settings, 'ADAPTIVE_TEMPLATE_DIRECTORIES_DESKTOP', '')
TABLET = getattr(settings, 'ADAPTIVE_TEMPLATE_DIRECTORIES_TABLET', '')
MOBILE = getattr(settings, 'ADAPTIVE_TEMPLATE_DIRECTORIES_MOBILE', '')


def get_template_suffix():
    """
    Returns template suffix based on settings and user agent used.
    """
    request = get_current_request()

    suffix = DESKTOP

    if hasattr(request, 'tablet') and request.tablet:
        suffix = TABLET

    elif hasattr(request, 'mobile') and request.mobile:
        suffix = MOBILE

    return suffix


def get_template_dirs(template_dirs):
    """
    Returns list of templates suffixed with appropriate string
    based on settings and user agent.
    """

    suffix = get_template_suffix()

    suffixed_dirs = [os.path.join(d, suffix) for d in template_dirs]

    return suffixed_dirs + list(template_dirs)
